@@ -8,15 +8,22 @@
 
 import unittest
 
-from couchdb.tests import client, couch_tests, schema, view
+from couchdb.tests import client, couch_tests, design, http, multipart, \
+                          mapping, view, package
+
 
 def suite():
     suite = unittest.TestSuite()
     suite.addTest(client.suite())
-    suite.addTest(couch_tests.suite())
-    suite.addTest(schema.suite())
+    suite.addTest(design.suite())
+    suite.addTest(http.suite())
+    suite.addTest(multipart.suite())
+    suite.addTest(mapping.suite())
     suite.addTest(view.suite())
+    suite.addTest(couch_tests.suite())
+    suite.addTest(package.suite())
     return suite
 
+
 if __name__ == '__main__':
     unittest.main(defaultTest='suite')
