@@ -32,12 +32,18 @@ def run(input=sys.stdin, output=sys.stdout):
     """
     functions = []
 
+    def _writejson(obj):
+        obj = json.encode(obj)
+        if isinstance(obj, unicode):
+            obj = obj.encode('utf-8')
+        output.write(obj)
+        output.write('\n')
+        output.flush()
+
     def _log(message):
         if not isinstance(message, basestring):
             message = json.encode(message)
-        output.write(json.encode({'log': message}))
-        output.write('\n')
-        output.flush()
+        _writejson({'log': message})
 
     def reset(config=None):
         del functions[:]
@@ -80,7 +86,7 @@ def run(input=sys.stdin, output=sys.stdout):
 
     def reduce(*cmd, **kwargs):
         code = BOM_UTF8 + cmd[0][0].encode('utf-8')
-        args = cmd[1:][0]
+        args = cmd[1]
         globals_ = {}
         try:
             exec code in {'log': _log}, globals_
@@ -116,6 +122,7 @@ def run(input=sys.stdin, output=sys.stdout):
         return [True, [results]]
 
     def rereduce(*cmd):
+        # Note: weird kwargs is for Python 2.5 compat
         return reduce(*cmd, **{'rereduce': True})
 
     handlers = {'reset': reset, 'add_fun': add_fun, 'map_doc': map_doc,
@@ -135,9 +142,7 @@ def run(input=sys.stdin, output=sys.stdout):
             else:
                 retval = handlers[cmd[0]](*cmd[1:])
                 log.debug('Returning  %r', retval)
-                output.write(json.encode(retval))
-                output.write('\n')
-                output.flush()
+                _writejson(retval)
     except KeyboardInterrupt:
         return 0
     except Exception, e:
